package main

import (
	"bufio"
	"crypto/sha512"
	"encoding/base32"
	"encoding/json"
	"errors"
	"log"
	"net/http"
	"os"
	"path"
	"strings"

	"srs.epita.fr/fic-server/libfic"
)

type uploadedFile struct {
	URI string
}

func createExerciceFile(theme fic.Theme, exercice fic.Exercice, args []string, body []byte) (interface{}, error) {
	var uf uploadedFile
	if err := json.Unmarshal(body, &uf); err != nil {
		return nil, err
	}

	if len(uf.URI) == 0 {
		return nil, errors.New("URI not filled")
	}

	hash := sha512.Sum512([]byte(uf.URI))
	pathname := path.Join(fic.FilesDir, strings.ToLower(base32.StdEncoding.EncodeToString(hash[:])), path.Base(uf.URI))

	if _, err := os.Stat(pathname); os.IsNotExist(err) {
		log.Println("Import file from Cloud:", uf.URI, "=>", pathname)
		if err := os.MkdirAll(path.Dir(pathname), 0777); err != nil {
			return nil, err
		} else if err := getCloudFile(uf.URI, pathname); err != nil {
			return nil, err
		}
	}

	return exercice.ImportFile(pathname, uf.URI)
}

func getCloudFile(pathname string, dest string) error {
	client := http.Client{}
	if req, err := http.NewRequest("GET", CloudDAVBase+pathname, nil); err != nil {
		return err
	} else {
		req.SetBasicAuth(CloudUsername, CloudPassword)
		if resp, err := client.Do(req); err != nil {
			return err
		} else {
			defer resp.Body.Close()

			if fd, err := os.Create(dest); err != nil {
				return err
			} else {
				defer fd.Close()

				if resp.StatusCode != http.StatusOK {
					return errors.New(resp.Status)
				} else {
					writer := bufio.NewWriter(fd)
					reader := bufio.NewReader(resp.Body)
					reader.WriteTo(writer)
					writer.Flush()
				}
			}
		}
	}
	return nil
}
