package main

import (
	"encoding/json"
	"fmt"
	"strconv"
	"strings"

	"srs.epita.fr/fic-server/libfic"
)

var ApiTeamsRouting = map[string]DispatchFunction{
	"GET":    listTeam,
	"PUT":    creationTeamMembers,
	"POST":   creationTeam,
	"DELETE": deletionTeam,
}

func nginxGenTeam() (string, error) {
	if teams, err := fic.GetTeams(); err != nil {
		return "", err
	} else {
		ret := ""
		for _, team := range teams {
			ret += fmt.Sprintf("        if ($ssl_client_s_dn ~ \"/C=FR/ST=France/O=Epita/OU=SRS/CN=%s\") { set $team \"%s\"; }\n", team.InitialName, team.InitialName)
		}

		return ret, nil
	}
}

func bindingTeams() (string, error) {
	if teams, err := fic.GetTeams(); err != nil {
		return "", err
        } else {
	        ret := ""
	        for _, team := range teams {
		        if members, err := team.GetMembers(); err != nil {
			        return "", err
			} else {
			        var mbs []string
				for _, member := range members {
				   mbs = append(mbs, fmt.Sprintf("%s %s", member.Firstname, member.Lastname))
				}
		                ret += fmt.Sprintf("%d;%s;%s\n", team.Id, team.Name, strings.Join(mbs, ";"))
			}
		}
		return ret, nil
	}
}

type uploadedTeam struct {
	Name  string
	Color uint32
}

type uploadedMember struct {
	Firstname string
	Lastname  string
	Nickname  string
	Company   string
}

func listTeam(args []string, body []byte) (interface{}, error) {
	if len(args) >= 2 {
		var team *fic.Team
		if tid, err := strconv.Atoi(args[0]); err != nil {
			if t, err := fic.GetTeamByInitialName(args[0]); err != nil {
				return nil, err
			} else {
				team = &t
			}
		} else {
			if tid == 0 {
				team = nil
			} else if t, err := fic.GetTeam(tid); err != nil {
				return nil, err
			} else {
				team = &t
			}

		}

		if args[1] == "my.json" {
			return fic.MyJSONTeam(team, true)
		} else if args[1] == "wait.json" {
			return fic.MyJSONTeam(team, false)
		} else if team != nil && args[1] == "members" {
			return team.GetMembers()
		} else if args[1] == "certificate" && team != nil {
			return CertificateAPI(*team, args[2:])
		} else if team != nil && args[1] == "name" {
			return team.Name, nil
		}
	} else if len(args) == 1 {
		if args[0] == "teams.json" {
			return fic.ExportTeams()
		} else if args[0] == "nginx" {
			return nginxGenTeam()
		} else if args[0] == "binding" {
			return bindingTeams()
		} else if tid, err := strconv.Atoi(string(args[0])); err != nil {
			return fic.GetTeamByInitialName(args[0])
		} else if team, err := fic.GetTeam(tid); err != nil {
			return nil, err
		} else {
			return team, nil
		}
	} else if len(args) == 0 {
		// List all teams
		return fic.GetTeams()
	}
	return nil, nil
}

func creationTeam(args []string, body []byte) (interface{}, error) {
	if len(args) == 1 {
		// List given team
		if tid, err := strconv.Atoi(string(args[0])); err != nil {
			return nil, err
		} else if team, err := fic.GetTeam(tid); err != nil {
			return nil, err
		} else {
			var members []uploadedMember
			if err := json.Unmarshal(body, &members); err != nil {
				return nil, err
			}

			for _, member := range members {
				team.AddMember(member.Firstname, member.Lastname, member.Nickname, member.Company)
			}

			return team.GetMembers()
		}
	} else if len(args) == 0 {
		// Create a new team
		var ut uploadedTeam
		if err := json.Unmarshal(body, &ut); err != nil {
			return nil, err
		}

		return fic.CreateTeam(ut.Name, ut.Color)
	} else {
		return nil, nil
	}
}

func creationTeamMembers(args []string, body []byte) (interface{}, error) {
	if len(args) == 1 {
		// List given team
		if tid, err := strconv.Atoi(string(args[0])); err != nil {
			return nil, err
		} else if team, err := fic.GetTeam(tid); err != nil {
			return nil, err
		} else {
			var member uploadedMember
			if err := json.Unmarshal(body, &member); err != nil {
				return nil, err
			}

			team.AddMember(member.Firstname, member.Lastname, member.Nickname, member.Company)

			return team.GetMembers()
		}
	} else if len(args) == 0 {
		// Create a new team
		var members []uploadedMember
		if err := json.Unmarshal(body, &members); err != nil {
			return nil, err
		}

		if team, err := fic.CreateTeam("", 0); err != nil {
			return nil, err
		} else {
			for _, member := range members {
				if _, err := team.AddMember(member.Firstname, member.Lastname, member.Nickname, member.Company); err != nil {
					return nil, err
				}
			}

			return team, nil
		}
	} else {
		return nil, nil
	}
}

func deletionTeam(args []string, body []byte) (interface{}, error) {
	if len(args) == 1 {
		if tid, err := strconv.Atoi(string(args[0])); err != nil {
			return nil, err
		} else if team, err := fic.GetTeam(tid); err != nil {
			return nil, err
		} else {
			return team.Delete()
		}
	} else {
		return nil, nil
	}
}
