#!/bin/sh

BASEURL="http://localhost:8081"
BASEURI="https://srs.epita.fr/owncloud/remote.php/webdav/FIC 2016"
BASEFILE="/files"
CLOUDPASS=fic:'f>t\nV33R|(+?$i*'

new_theme() {
    NAME=`echo $1 | sed 's/"/\\\\"/g'`
    AUTHORS=`echo $2 | sed 's/"/\\\\"/g'`
    curl -f -s -d "{\"name\": \"$NAME\", \"authors\": \"$AUTHORS\"}" "${BASEURL}/api/themes/" |
	grep -Eo '"id":[0-9]+,' | grep -Eo "[0-9]+"
}

new_exercice() {
    THEME="$1"
    TITLE=`echo "$2" | sed 's/"/\\\\"/g'`
    STATEMENT=`echo "$3" | sed 's/"/\\\\"/g' | sed ':a;N;$!ba;s/\n/<br>/g'`
    HINT=`echo "$4" | sed 's/"/\\\\"/g' | sed ':a;N;$!ba;s/\n/<br>/g'`
    DEPEND="$5"
    GAIN="$6"
    VIDEO="$7"

    curl -f -s -d "{\"title\": \"$TITLE\", \"statement\": \"$STATEMENT\", \"hint\": \"$HINT\", \"depend\": $DEPEND, \"gain\": $GAIN, \"videoURI\": \"$VIDEO\"}" "${BASEURL}/api/themes/$THEME" |
	grep -Eo '"id":[0-9]+,' | grep -Eo "[0-9]+"
}

new_file() {
    THEME="$1"
    EXERCICE="$2"
    URI="$3"

    curl -f -s -d "{\"URI\": \"${BASEFILE}${URI}\"}" "${BASEURL}/api/themes/$THEME/$EXERCICE/files" |
	grep -Eo '"id":[0-9]+,' | grep -Eo "[0-9]+"
}

new_key() {
    THEME="$1"
    EXERCICE="$2"
    NAME="$3"
    KEY=`echo $4 | sed 's/"/\\\\"/g' | sed 's#\\\\#\\\\\\\\#g'`

    curl -f -s -d "{\"name\": \"$NAME\", \"key\": \"$KEY\"}" "${BASEURL}/api/themes/$THEME/$EXERCICE/keys" |
	grep -Eo '"id":[0-9]+,' | grep -Eo "[0-9]+"
}

# Theme
curl -f -s -X PROPFIND -u "${CLOUDPASS}" "${BASEURI}" | xmllint --format - | grep 'd:href' | sed -E 's/^.*>(.*)<.*$/\1/' | sed 1d | tac | while read f; do basename "$f"; done | while read THEME_URI
do
    THM_BASEURI="/${THEME_URI}/"
    THEME_NAME=$(echo "${THEME_URI}" | sed -E 's/%20/ /g' | sed -E 's/%c3%a9/é/g' | sed -E 's/%c3%a8/è/g')
    THEME_AUTHORS=$(curl -f -s -u "${CLOUDPASS}" "${BASEURI}${THM_BASEURI}/AUTHORS.txt" | sed 's/$/,/' | xargs)
    THEME_ID=`new_theme "$THEME_NAME" "$THEME_AUTHORS"`
    if [ -z "$THEME_ID" ]; then
        echo -e "\e[31;01m!!! An error occured during theme add\e[00m"
	continue
    else
        echo -e "\e[33m>>> New theme created:\e[00m $THEME_ID - $THEME_NAME"
    fi

    LAST=null
    EXO_NUM=0
    curl -f -s -X PROPFIND -u "${CLOUDPASS}" "${BASEURI}${THM_BASEURI}" | xmllint --format - | grep 'd:href' | sed -E 's/^.*>(.*)<.*$/\1/' | sed -E 's/%20/ /g' | sed -E 's/%c3%a9/é/g' | sed -E 's/%c3%a8/è/g' | sed 1d | while read f; do basename "$f"; done | while read EXO_NAME
    do
	if ! echo $EXO_NAME | grep Exercice > /dev/null
	then
	    continue
	fi

	EXO_NUM=$((EXO_NUM + 1))
	echo
	echo -e "\e[36m--- Filling exercice ${EXO_NUM} in theme ${THEME_NAME}\e[00m"

	EXO_BASEURI="${EXO_NAME}/"

	FILES=$(curl -f -s -X PROPFIND -u "${CLOUDPASS}" "${BASEURI}${THM_BASEURI}${EXO_BASEURI}" | xmllint --format - | grep 'd:href' | sed -E 's/^.*>(.*)<.*$/\1/' | sed 1d | while read f; do basename $f; done)

	EXO_VIDEO=$(echo "$FILES" | grep -E "\.(mov|mkv|mp4|avi|flv|ogv|webm)$" | tail -1)

	if [ "${LAST}" = "null" ]; then
	    echo ">>> Assuming this exercice has no dependency"
	else
	    echo ">>> Assuming this exercice depends on the last entry (id=${LAST})"
	fi

	EXO_GAIN=$((3 * (2 ** $EXO_NUM) - 1))
	echo ">>> Using default gain: ${EXO_GAIN} points"

	EXO_DESC=$(curl -f -s -u "${CLOUDPASS}" "${BASEURI}${THM_BASEURI}${EXO_BASEURI}/description.txt")
	EXO_HINT=$(curl -f -s -u "${CLOUDPASS}" "${BASEURI}${THM_BASEURI}${EXO_BASEURI}/hint.txt")

	EXO_ID=`new_exercice "${THEME_ID}" "${EXO_NAME}" "${EXO_DESC}" "${EXO_HINT}" "${LAST}" "${EXO_GAIN}" "${THM_BASEURI}${EXO_BASEURI}${EXO_VIDEO}"`
	if [ -z "$EXO_ID" ]; then
	    echo -e "\e[31;01m!!! An error occured during exercice add.\e[00m"
	    continue
	else
	    echo -e "\e[32m>>> New exercice created:\e[00m $EXO_ID - $EXO_NAME"
        fi


	# Keys
	curl -f -s -u "${CLOUDPASS}" "${BASEURI}${THM_BASEURI}${EXO_BASEURI}/keys.txt" | while read KEYLINE
	do
	    KEY_NAME=$(echo "$KEYLINE" | cut -d : -f 1)
	    KEY_RAW=$(echo "$KEYLINE" | cut -d : -f 2-)

	    if [ -z "${KEY_NAME}" ]; then
		KEY_NAME="Flag"
	    fi

	    KEY_ID=`new_key "${THEME_ID}" "${EXO_ID}" "${KEY_NAME}" "${KEY_RAW}"`
	    if [ -z "$KEY_ID" ]; then
	        echo -e "\e[31;01m!!! An error occured during key import!\e[00m (name=${KEYNAME};raw=${KEY_RAW})"
	    else
	        echo -e "\e[32m>>> New key added:\e[00m $KEY_ID - $KEY_NAME"
	    fi
	done


	# Files
	for f in $FILES; do echo $f; done | grep -vEi "(ressources|readme|description.txt|hint.txt|keys.txt|${EXO_VIDEO})" |
	    while read FBASE
	do
	    echo "Import file ${BASEURI}${THM_BASEURI}${EXO_BASEURI}${FBASE}"
	    FILE_ID=`new_file "${THEME_ID}" "${EXO_ID}" "${THM_BASEURI}${EXO_BASEURI}${FBASE}"`
	    if [ -z "$FILE_ID" ]; then
	        echo -e "\e[31;01m!!! An error occured during file import! Please check path.\e[00m"
	    else
	        echo -e "\e[32m>>> New file added:\e[00m $FILE_ID - $FBASE"
	    fi
	done


	LAST=$EXO_ID
    done
    echo
done
