package fic

import (
	"encoding/hex"
	"fmt"
	"time"
)

type myTeamFile struct {
	Path     string `json:"path"`
	Name     string `json:"name"`
	Checksum string `json:"checksum"`
	Size     int64  `json:"size"`
}
type myTeamExercice struct {
	ThemeId      int          `json:"theme_id"`
	Statement    string       `json:"statement"`
	Hint         string       `json:"hint"`
	Gain         int64        `json:"gain"`
	Files        []myTeamFile `json:"files"`
	Keys         []string     `json:"keys"`
	Solved       bool         `json:"solved"`
	SolvedTime   time.Time    `json:"solved_time"`
	SolvedNumber int64        `json:"solved_number"`
	VideoURI     string       `json:"video_uri"`
}
type myTeam struct {
	Id        int64                     `json:"team_id"`
	Name      string                    `json:"name"`
	Points    int64                     `json:"score"`
	Members   []Member                  `json:"members"`
	Exercices map[string]myTeamExercice `json:"exercices"`
}

func MyJSONTeam(t *Team, started bool) (interface{}, error) {
	ret := myTeam{}
	if t == nil {
		ret.Id = 0
	} else {
		ret.Name = t.Name
		ret.Id = t.Id
		ret.Points, _ = t.GetPoints()
		if members, err := t.GetMembers(); err == nil {
			ret.Members = members
		}

	}
	ret.Exercices = map[string]myTeamExercice{}

	if exos, err := GetExercices(); err != nil {
		return ret, err
	} else if started {
		for _, e := range exos {
			if t == nil || t.HasAccess(e) {
				exercice := myTeamExercice{}
				if tid, err := e.GetThemeId(); err == nil {
					exercice.ThemeId = tid
				}
				exercice.Statement = e.Statement
				exercice.Hint = e.Hint
				if t == nil {
					exercice.VideoURI = e.VideoURI
					exercice.Solved = true
					exercice.SolvedNumber = e.TriedCount()
				} else {
					exercice.Solved, exercice.SolvedTime, exercice.SolvedNumber = t.HasSolved(e)
				}

				exercice.Keys = []string{}

				if keys, err := e.GetKeys(); err != nil {
					return nil, err
				} else {
					for _, k := range keys {
						if t == nil {
							exercice.Keys = append(exercice.Keys, fmt.Sprintf("%x", k.Value)+k.Type)
						} else {
							exercice.Keys = append(exercice.Keys, k.Type)
						}
					}
				}

				exercice.Files = []myTeamFile{}

				if files, err := e.GetFiles(); err != nil {
					return nil, err
				} else {
					for _, f := range files {
						exercice.Files = append(exercice.Files, myTeamFile{f.Path, f.Name, hex.EncodeToString(f.Checksum), f.Size})
					}
				}

				ret.Exercices[fmt.Sprintf("%d", e.Id)] = exercice
			}
		}
	}

	return ret, nil
}
