package fic

import (
	"fmt"
)

type Theme struct {
	Id      int64  `json:"id"`
	Name    string `json:"name"`
	Authors string `json:"authors"`
}

func GetThemes() ([]Theme, error) {
	if rows, err := DBQuery("SELECT id_theme, name, authors FROM themes"); err != nil {
		return nil, err
	} else {
		defer rows.Close()

		var themes = make([]Theme, 0)
		for rows.Next() {
			var t Theme
			if err := rows.Scan(&t.Id, &t.Name, &t.Authors); err != nil {
				return nil, err
			}
			themes = append(themes, t)
		}
		if err := rows.Err(); err != nil {
			return nil, err
		}

		return themes, nil
	}
}

func GetTheme(id int) (Theme, error) {
	var t Theme
	if err := DBQueryRow("SELECT id_theme, name, authors FROM themes WHERE id_theme=?", id).Scan(&t.Id, &t.Name, &t.Authors); err != nil {
		return t, err
	}

	return t, nil
}

func CreateTheme(name string, authors string) (Theme, error) {
	if res, err := DBExec("INSERT INTO themes (name, authors) VALUES (?, ?)", name, authors); err != nil {
		return Theme{}, err
	} else if tid, err := res.LastInsertId(); err != nil {
		return Theme{}, err
	} else {
		return Theme{tid, name, authors}, nil
	}
}

func (t Theme) Update() (int64, error) {
	if res, err := DBExec("UPDATE themes SET name = ?, authors = ? WHERE id_theme = ?", t.Name, t.Authors, t.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func (t Theme) Delete() (int64, error) {
	if res, err := DBExec("DELETE FROM themes WHERE id_theme = ?", t.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

type ExportedExercice struct {
	Title  string `json:"title"`
	Gain   int64  `json:"gain"`
	Solved int64  `json:"solved"`
	Tried  int64  `json:"tried"`
}

type exportedTheme struct {
	Name      string                      `json:"name"`
	Authors   string                      `json:"authors"`
	Exercices map[string]ExportedExercice `json:"exercices"`
}

func ExportThemes() (interface{}, error) {
	if themes, err := GetThemes(); err != nil {
		return nil, err
	} else {
		ret := map[string]exportedTheme{}
		for _, theme := range themes {
			if exercices, err := theme.GetExercices(); err != nil {
				return nil, err
			} else {
				exos := map[string]ExportedExercice{}
				for _, exercice := range exercices {
					exos[fmt.Sprintf("%d", exercice.Id)] = ExportedExercice{
						exercice.Title,
						exercice.Gain,
						exercice.SolvedCount(),
						exercice.TriedTeamCount(),
					}
				}
				ret[fmt.Sprintf("%d", theme.Id)] = exportedTheme{
					theme.Name,
					theme.Authors[:len(theme.Authors)-1],
					exos,
				}
			}
		}

		return ret, nil
	}
}
